@@ -58,8 +58,7 @@ Gives the version (or language) information for this subtable
 
 =item val
 
-This points to a L<Font::TTF::Segarr> which contains the content of the particular
-subtable.
+A hash keyed by the codepoint value (not a string) storing the glyph id
 
 =back
 
@@ -69,8 +68,8 @@ subtable.
 
 use strict;
 use vars qw(@ISA);
-require Font::TTF::Table;
-require Font::TTF::Segarr;
+use Font::TTF::Table;
+use Font::TTF::Utils;
 
 @ISA = qw(Font::TTF::Table);
 
@@ -114,9 +113,9 @@ sub read
         $s->{'Ver'} = $ver;
         if ($form == 0)
         {
-            $s->{'val'} = Font::TTF::Segarr->new;
+            my ($j) = 0;
             $fh->read($dat, 256);
-            $s->{'val'}->fastadd_segment(0, 2, unpack("C*", $dat));
+            $s->{'val'} = {map {$j++; ($_ ? ($j - 1, $_) : ())} unpack("C*", $dat)};
             $s->{'Start'} = 0;
             $s->{'Num'} = 256;
         } elsif ($form == 6)
@@ -126,10 +125,9 @@ sub read
             $fh->read($dat, 4);
             ($start, $ecount) = unpack("n2", $dat);
             $fh->read($dat, $ecount << 1);
-            $s->{'val'} = Font::TTF::Segarr->new;
-            $s->{'val'}->fastadd_segment($start, 2, unpack("n*", $dat));
             $s->{'Start'} = $start;
             $s->{'Num'} = $ecount;
+            $s->{'val'} = {map {$start++; ($_ ? ($start - 1, $_) : ())} unpack("n*", $dat)};
         } elsif ($form == 2)
         {
 # no idea what to do here yet
@@ -139,7 +137,6 @@ sub read
             $num = unpack("n", $dat);
             $num >>= 1;
             $fh->read($dat, $len - 14);
-            $s->{'val'} = Font::TTF::Segarr->new;
             for ($j = 0; $j < $num; $j++)
             {
                 $end = unpack("n", substr($dat, $j << 1, 2));
@@ -147,7 +144,7 @@ sub read
                 $delta = unpack("n", substr($dat, ($j << 1) + ($num << 2) + 2, 2));
                 $delta -= 65536 if $delta > 32767;
                 $range = unpack("n", substr($dat, ($j << 1) + $num * 6 + 2, 2));
-                @ids = ();
+                $s->{'Start'} = $start unless $j;
                 for ($k = $start; $k <= $end; $k++)
                 {
                     if ($range == 0)
@@ -155,14 +152,36 @@ sub read
                     else
                     { $id = unpack("n", substr($dat, ($j << 1) + $num * 6 +
                                         2 + ($k - $start) * 2 + $range, 2)) + $delta; }
-		            $id -= 65536 if $id > 65536;
-                    push (@ids, $id);
+		            $id -= 65536 if $id >= 65536;
+                    $s->{'val'}{$k} = $id if ($id);
                 }
-                $s->{'val'}->fastadd_segment($start, 0, @ids);
             }
-            $s->{'val'}->tidy;
             $s->{'Num'} = 0x10000;               # always ends here
-            $s->{'Start'} = $s->{'val'}[0]{'START'};
+        } elsif ($form == 8 || $form == 10)
+        {
+            if ($form == 8)
+            {
+                $fh->read($dat, 8196);
+                $num = unpack("N", substr($dat, 8192, 4)); # don't need the map
+            } else
+            {
+                $fh->read($dat, 4);
+                $num = unpack("N", $dat);
+            }
+            $fh->read($dat, 12 * $num);
+            for ($j = 0; $j < $num; $j++)
+            {
+                ($start, $end, $s) = unpack("N3", substr($dat, $j * 12, 12));
+                for ($k = $start; $k <= $end; $k++)
+                { $s->{'val'}{$k} = $s++; }
+            }
+        } elsif ($form == 12)
+        {
+            $fh->read($dat, 8);
+            ($start, $num) = unpack("N2", $dat);
+            $fh->read($dat, $num << 1);
+            for ($j = 0; $j < $num; $j++)
+            { $s->{'val'}{$start + $j} = unpack("n", substr($dat, $j << 1, 2)); }
         }
     }
     $self;
@@ -181,7 +200,7 @@ sub ms_lookup
     my ($self, $uni) = @_;
 
     $self->find_ms || return undef unless (defined $self->{' mstable'});
-    return $self->{' mstable'}{'val'}->at($uni);
+    return $self->{' mstable'}{'val'}{$uni};
 }
 
 
@@ -222,7 +241,7 @@ just copies from input file to output
 sub out
 {
     my ($self, $fh) = @_;
-    my ($loc, $s, $i, $base_loc, $j);
+    my ($loc, $s, $i, $base_loc, $j, @keys);
 
     return $self->SUPER::out($fh) unless $self->{' read'};
 
@@ -235,49 +254,53 @@ sub out
     for ($i = 0; $i < $self->{'Num'}; $i++)
     {
         $s = $self->{'Tables'}[$i];
-        $s->{'val'}->tidy;
+        @keys = sort {$a <=> $b} keys %{$s->{'val'}};
         $s->{' outloc'} = $fh->tell();
         $fh->print(pack("n3", $s->{'Format'}, 0, $s->{'Ver'}));       # come back for length
         if ($s->{'Format'} == 0)
         {
-            $fh->print(pack("C256", $s->{'val'}->at(0, 256)));
+            $fh->print(pack("C256", @{$s->{'val'}}{0 .. 255}));
         } elsif ($s->{'Format'} == 6)
         {
             $fh->print(pack("n2", $s->{'Start'}, $s->{'Num'}));
-            $fh->print(pack("n*", $s->{'val'}->at($s->{'Start'}, $s->{'Num'})));
+            $fh->print(pack("n*", @{$s->{'val'}}{$s->{'Start'} .. $s->{'Start'} + $s->{'Num'} - 1}));
         } elsif ($s->{'Format'} == 2)
         {
         } elsif ($s->{'Format'} == 4)
         {
-            my ($num, $sRange, $eSel);
-            my (@deltas, $delta, @range, $flat, $k, $segs, $count);
+            my ($num, $sRange, $eSel, $eShift, @starts, @ends, $doff);
+            my (@deltas, $delta, @range, $flat, $k, $segs, $count, $newseg, $v);
 
-            $num = $#{$s->{'val'}} + 1;
-            $segs = $s->{'val'};
-            for ($sRange = 1, $eSel = 0; $sRange <= $num; $eSel++)
-            { $sRange <<= 1;}
-            $eSel--;
-            $fh->print(pack("n4", $num * 2, $sRange, $eSel, ($num * 2) - $sRange));
-            $fh->print(pack("n*", map {$_->{'START'} + $_->{'LEN'} - 1} @$segs));
-            $fh->print(pack("n", 0));
-            $fh->print(pack("n*", map {$_->{'START'}} @$segs));
-
-            for ($j = 0; $j < $num; $j++)
+            push(@keys, 0xFFFF) unless ($keys[-1] == 0xFFFF);
+            $newseg = 1; $num = 0;
+            for ($j = 0; $j <= $#keys; $j++)
             {
-                $delta = $segs->[$j]{'VAL'}[0]; $flat = 1;
-                for ($k = 1; $k < $segs->[$j]{'LEN'}; $k++)
+                $v = $s->{'val'}{$keys[$j]};
+                if ($newseg)
                 {
-                    if ($segs->[$j]{'VAL'}[$k] == 0)
-                    { $flat = 0; }
-                    if ($delta + $k != $segs->[$j]{'VAL'}[$k])
-                    {
-                        $delta = 0;
-                        last;
-                    }
+                    $delta = $v;
+                    $doff = $j;
+                    $flat = 1;
+                    push(@starts, $keys[$j]);
+                    $newseg = 0;
+                }
+                $delta = 0 if ($delta + $j - $doff != $v);
+                $flat = 0 if ($v == 0);
+                if ($j == $#keys || $keys[$j] + 1 != $keys[$j+1])
+                {
+                    push (@ends, $keys[$j]);
+                    push (@deltas, $delta ? $delta - $keys[$doff] : 0);
+                    push (@range, $flat);
+                    $num++;
+                    $newseg = 1;
                 }
-                push (@range, $flat);
-                push (@deltas, ($delta ? $delta - $segs->[$j]{'START'} : 0));
             }
+
+            ($num, $sRange, $eSel, $eShift) = Font::TTF::Utils::TTF_bininfo($num, 2);
+            $fh->print(pack("n4", $num * 2, $sRange, $eSel, $eShift));
+            $fh->print(pack("n*", @ends));
+            $fh->print(pack("n", 0));
+            $fh->print(pack("n*", @starts));
             $fh->print(pack("n*", @deltas));
 
             $count = 0;
@@ -289,7 +312,7 @@ sub out
                 else
                 {
                     $range[$j] = ($count + $num - $j) << 1;
-                    $count += $segs->[$j]{'LEN'};
+                    $count += $ends[$j] - $starts[$j] + 1;
                 }
             }
 
@@ -298,8 +321,7 @@ sub out
             for ($j = 0; $j < $num; $j++)
             {
                 next if ($range[$j] == 0);
-                for ($k = 0; $k < $segs->[$j]{'LEN'}; $k++)
-                { $fh->print(pack("n", $segs->[$j]{'VAL'}[$k])); }
+                $fh->print(pack("n*", @{$s->{'val'}}{$starts[$j] .. $ends[$j]}));
             }
         }
 
@@ -325,13 +347,10 @@ sub reverse
 {
     my ($self, $tnum) = @_;
     my ($table) = defined $tnum ? $self->{'Tables'}[$tnum] : $self->find_ms;
-    my (@res, $i, $s, $first);
+    my (@res, $code, $gid);
 
-    foreach $s (@{$table->{'val'}})
-    {
-        $first = $s->{'START'};
-        map {$res[$_] = $first unless $res[$_]; $first++;} @{$s->{'VAL'}};
-    }
+    while (($code, $gid) = each(%{$table->{'val'}}))
+    { $res[$gid] = $code unless ($res[$gid] > 0 && $res[$gid] < $code); }
     @res;
 }
 
@@ -339,13 +358,17 @@ sub reverse
 
 =head1 BUGS
 
+=over 4
+
 =item *
 
 No support for format 2 tables (MBCS)
 
+=back
+
 =head1 AUTHOR
 
-Martin Hosken E<Martin_Hosken@sil.org>. See L<Font::TTF::Font> for copyright and
+Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
 licensing.
 
 =cut
